import subprocess
import os
import shutil
import tempfile

from cleo import Command

from ..utils import replace_string_in_file


class PullCommand(Command):
    """
    Pull template project into given directory

    pull
        {--directory=? : By default current directory}
        {--repo=MasoniteFramework/cookie-cutter : Repository to pull from}
        {--branch=? : Repository branch to pull from, default is the default from repo}
        {--switch-branch=? : Branch to put modifications from}
        {--exclude=? : Override default files or folders to exclude (separated by ,)}
    """

    EXCLUDE = [
        "tests",
        ".git",
        "README.md",
        ".gitignore",
        ".env-example",
        ".env.testing",
        "craft",
        "makefile",
        "package.json",
        "pyproject.toml",
        "requirements.txt",
        "setup.cfg",
        "webpack.mix.js",
        "wsgi.py",
    ]

    def handle(self):
        cwd = os.getcwd()
        directory = os.path.abspath(self.option("directory")) or cwd
        repo = self.option("repo")
        branch = self.option("branch")

        switch_branch = self.option("switch-branch")
        excluded_files = self.option("exclude")

        git_clone_args = [
            "git",
            "clone",
            "--depth",
            "1",
            f"https://github.com/{repo}.git",
        ]
        if branch:
            git_clone_args.extend(["-b", branch])
        paths_to_remove = []
        if excluded_files:
            paths_to_remove = excluded_files.split(",")
        else:
            paths_to_remove = self.EXCLUDE

        repo_name = repo.split("/")[-1]
        with tempfile.TemporaryDirectory() as tmp_dir:
            source_dir = os.path.join(tmp_dir, repo_name)
            os.chdir(tmp_dir)
            # Clone into temporary dir
            subprocess.run(git_clone_args, check=True, stdout=subprocess.PIPE).stdout

            # # remove files or folders to ignore
            for path in paths_to_remove:
                abs_path = os.path.join(source_dir, path)
                try:
                    if os.path.isdir(abs_path):
                        shutil.rmtree(abs_path)
                    else:
                        os.remove(abs_path)
                except FileNotFoundError:
                    pass

            # if all good
            os.chdir(cwd)
            if switch_branch:
                subprocess.run(
                    ["git", "checkout", "-b", switch_branch],
                    check=True,
                    stdout=subprocess.PIPE,
                ).stdout

            # copy all files
            shutil.copytree(source_dir, directory, dirs_exist_ok=True)

        # convert project to work inside given folder
        if directory:
            replace_string_in_file(
                os.path.join(directory, "config/auth.py"),
                "from app.",
                "from tests.integrations.app.",
            )

            replace_string_in_file(
                os.path.join(directory, "databases/seeds/user_table_seeder.py"),
                "from app.",
                "from tests.integrations.app.",
            )

            replace_string_in_file(
                os.path.join(directory, "config/filesystem.py"),
                "storage/",
                "tests/integrations/storage/",
            )

            # Kernel:
            kernel = os.path.join(directory, "Kernel.py")
            tokens = [
                ("from app.", "from tests.integrations.app."),
                (
                    'self.application.bind("config.location", "config")',
                    'self.application.bind("config.location", "tests/integrations/config")',
                ),
                ("app/", "tests/integrations/app/"),
                ("routes/web", "tests/integrations/routes/web"),
                ("resources/", "tests/integrations/resources/"),
                ("databases/", "tests/integrations/databases/"),
                ("templates/", "tests/integrations/templates/"),
            ]
            for search, replace in tokens:
                replace_string_in_file(kernel, search, replace)

        self.line("Pulled !")



def replace_string_in_file(filepath, searched, replaced):
    with open(filepath) as f:
        file_source = f.read()
        # replace all occurences
        replace_string = file_source.replace(searched, replaced)

    with open(filepath, "w") as f:
        f.write(replace_string)



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="masonite-package-sync",
    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version="0.5.0",
    package_dir={"": "src"},
    description="Masonite package synchronisation",
    long_description=long_description,
    long_description_content_type="text/markdown",
    # The project's main homepage.
    url="https://github.com/girardinsamuel/masonite-package-sync",
    # Author details
    author="Samuel Girardin",
    author_email="sam@masoniteproject.com",
    # Choose your license
    license="MIT",
    # List run-time dependencies here
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=[
        "cleo>=0.8.1,<0.9",
    ],
    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        "Development Status :: 4 - Beta",
        # Indicate who your project is intended for
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "Environment :: Web Environment",
        # Pick your license as you wish (should match "license" above)
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    # What does your project relate to?
    keywords="Masonite, MasoniteFramework, Python, Packages",
    extras_require={
        "test": [
            "coverage",
            "black",
            "flake8",
            "pytest",
            "twine",
            "bump2version",
        ],
    },
    packages=find_packages(where="src"),
    entry_points={
        "console_scripts": [
            "masonite-package = masonite_package_sync.main:application.run",
        ],
    },
)
